@@ -8,7 +8,7 @@
 # This library is distributed under the terms of the GNU Public License (GPL)
 # for full details see the file COPYING
 #
-# $Id: load.instruments.R 1267 2012-11-27 00:21:50Z gsee $
+# $Id: load.instruments.R 1656 2014-11-28 01:07:57Z gsee $
 #
 ###############################################################################
 
@@ -27,6 +27,8 @@
 #' 
 #' You will need to specify a \code{currency}, unless the instrument \code{type} is 'currency'
 #' 
+#' Use the \code{identifier_cols} argument to specify which fields (if any) in the CSV are to be passed to \code{\link{instrument}} as the \code{identifiers} argument
+#'
 #' Typically, columns will exist for \code{multiplier} and \code{tick_size}.
 #' 
 #' Any other columns necessary to define the specified instrument type will also be required to avoid fatal Errors.  
@@ -38,6 +40,8 @@
 #' @param metadata optional, data.frame containing metadata, default NULL, see Details
 #' @param id_col numeric column containing id if primary_id isn't defined, default 1
 #' @param default_type character string to use as instrument type fallback, see Details
+#' @param identifier_cols character vector of field names to be passed as identifiers, see Details
+#' @param overwrite TRUE/FALSE. See \code{\link{instrument}}.
 #' @seealso 
 #' \code{\link{loadInstruments}},
 #' \code{\link{instrument}}, 
@@ -52,7 +56,7 @@
 #'
 #' }
 #' @export
-load.instruments <- function (file=NULL, ..., metadata=NULL, id_col=1, default_type='stock') {
+load.instruments <- function (file=NULL, ..., metadata=NULL, id_col=1, default_type='stock', identifier_cols=NULL, overwrite=TRUE) {
 
     if(is.null(file) && is.null(metadata)) stop("You must pass either a file identifier string or a metadata object to be converted.")
     if(is.null(metadata)){
@@ -87,51 +91,55 @@ load.instruments <- function (file=NULL, ..., metadata=NULL, id_col=1, default_t
     
     #now process the data
     for(rn in 1:nrow(filedata)){
-        if(!isTRUE(is.instrument(try(getInstrument(as.character(filedata[rn,id_col]),silent=TRUE),silent=TRUE)))){
-            type=as.character(filedata[rn,'type'])
-            arg<-as.list(filedata[rn,])
-            if(type=='spread' || type=='guaranteed_spread'){
-				if(!is.null(arg$members)){
-					arg$members<-unlist(strsplit(arg$members,','))
-				}
-				if(!is.null(arg$memberratio)){
-					arg$memberratio<-unlist(strsplit(arg$memberratio,','))
-				}
-				if(!is.null(arg$ratio)){
-					arg$memberratio<-unlist(strsplit(arg$ratio,','))
-				}
-			}
-            arg$type<-NULL
-            arg<-arg[!is.na(arg)]
-            arg<-arg[!arg==""]
-            if (set_primary) {
-                arg$primary_id<-filedata[rn,id_col]
+        type <- as.character(filedata[rn,'type'])
+        arg <- as.list(filedata[rn,])
+        if(type=='spread' || type=='guaranteed_spread'){
+            if(!is.null(arg$members)){
+                arg$members<-unlist(strsplit(arg$members,','))
             }
+            if(!is.null(arg$memberratio)){
+                arg$memberratio<-unlist(strsplit(arg$memberratio,','))
+            }
+            if(!is.null(arg$ratio)){
+                arg$memberratio<-unlist(strsplit(arg$ratio,','))
+            }
+        }
+        arg$type <- NULL
+        arg <- arg[!is.na(arg)]
+        arg <- arg[!arg==""]
+        if (set_primary) {
+            arg$primary_id<-filedata[rn,id_col]
+        }
+        
+        #do some name cleanup to make up for Reuters silliness
+        if(substr(arg$primary_id,1,1)==1) arg$primary_id <- substr(arg$primary_id,2,nchar(arg$primary_id))
+        arg$primary_id<-make.names(arg$primary_id)
+        if(!is.null(arg$X.RIC)){
+            if(substr(arg$X.RIC,1,1)==1) arg$X.RIC <- substr(arg$X.RIC,2,nchar(arg$X.RIC))
+        }            
+        if(!is.null(arg$RIC)){
+            if(substr(arg$RIC,1,1)==1) arg$RIC <- substr(arg$RIC,2,nchar(arg$RIC))
+        }            
+        if(length(dotargs)) arg<-c(arg,dotargs)
+        
+        if(!is.null(identifier_cols) && any(identifier_cols %in% names(arg))){
+            arg$identifiers <- arg[names(arg) %in% identifier_cols]
+            arg[identifier_cols] <- NULL
+        }
+        
+        arg$overwrite <- overwrite
+        if(is.function(try(match.fun(type),silent=TRUE))){
+            out <- try(do.call(type,arg))
             
-            #do some name cleanup to make up for Reuters silliness
-            if(substr(arg$primary_id,1,1)==1) arg$primary_id <- substr(arg$primary_id,2,nchar(arg$primary_id))
-            arg$primary_id<-make.names(arg$primary_id)
-            if(!is.null(arg$X.RIC)){
-                if(substr(arg$X.RIC,1,1)==1) arg$X.RIC <- substr(arg$X.RIC,2,nchar(arg$X.RIC))
-            }            
-            if(!is.null(arg$RIC)){
-                if(substr(arg$RIC,1,1)==1) arg$RIC <- substr(arg$RIC,2,nchar(arg$RIC))
-            }            
-            if(length(dotargs)) arg<-c(arg,dotargs)
             
-            if(is.function(try(match.fun(type),silent=TRUE))){
-                out <- try(do.call(type,arg))
-                #TODO recover gracefully?
-            } else {
-                # the call for a function named for type didn't work, so we'll try calling instrument as a generic
-				type=c(type,"instrument")
-				arg$type<-type # set the type
-                arg$assign_i<-TRUE # assign to the environment
-				try(do.call("instrument",arg))
-			}
-        } else {   
-            warning(filedata[rn,id_col]," already exists in the .instrument environment")
-        } # end instrument check
+            #TODO recover gracefully?
+        } else {
+            # the call for a function named for type didn't work, so we'll try calling instrument as a generic
+            type=c(type,"instrument")
+            arg$type<-type # set the type
+            arg$assign_i<-TRUE # assign to the environment
+            try(do.call("instrument",arg))
+        }
     } # end loop on rows
 }
 
@@ -173,7 +181,7 @@ setSymbolLookup.FI<-function(base_dir, Symbols, ..., split_method=c("days","comm
 
     #load all instrument names
     instr_names <- if(missing(Symbols)) {
-        ls_non_currencies(ls(pos=FinancialInstrument:::.instrument)) #if roots begin with a dot, this will filter out roots and currencies
+        ls_non_currencies(ls(pos=.instrument)) #if roots begin with a dot, this will filter out roots and currencies
     } else Symbols
     
     #TODO add check to make sure that src is actually the name of a getSymbols function
@@ -202,7 +210,7 @@ setSymbolLookup.FI<-function(base_dir, Symbols, ..., split_method=c("days","comm
         symbol<-list()
         symbol[[1]]<-params
         # construct $dir
-        symbol[[1]]$dir<-paste(base_dir,instr_str,sep=sepch)
+        symbol[[1]]$dir<-base_dir
         names(symbol)[1]<-instr
         new.symbols<-c(new.symbols,symbol)
     }
@@ -237,10 +245,7 @@ setSymbolLookup.FI<-function(base_dir, Symbols, ..., split_method=c("days","comm
 #' value for an argument, that value will be used.  If the user did not provide
 #' a value for an argument, but there is a value for that argument for the 
 #' given \code{Symbol} in the Symbol Lookup Table (see 
-#' \code{\link{setSymbolLookup.FI}}), that value will be used.  Otherwise,
-#' if \code{\link[Defaults]{setDefaults}} has been applied to this function 
-#' (\code{getSymbols.FI}) for the argument, the value that was set with 
-#' \code{setDefaults} will be used.  Finally, if none of the above are true,
+#' \code{\link{setSymbolLookup.FI}}), that value will be used.  Otherwise, 
 #' the formal defaults will be used.
 #'
 #' @param Symbols a character vector specifying the names of each symbol to be 
@@ -282,19 +287,23 @@ setSymbolLookup.FI<-function(base_dir, Symbols, ..., split_method=c("days","comm
 #' }
 #' @export
 getSymbols.FI <- function(Symbols,
-                            from='2010-01-01',
-                            to=Sys.Date(),
+                            from=getOption("getSymbols.FI.from", "2010-01-01"),
+                            to=getOption("getSymbols.FI.to", Sys.Date()),
                             ..., 
-                            dir="",
-                            return.class="xts",
-                            extension="rda",
-                            split_method = c("days", "common"),
-                            use_identifier = NA,
-                            date_format=NULL,
-                            verbose=TRUE,
-                            days_to_omit=c("Saturday", "Sunday"),
-                            indexTZ=NA
-                         ) 
+                            dir=getOption("getSymbols.FI.dir", ""),
+                            return.class=getOption("getSymbols.FI.return.class", 
+                                                   "xts"),
+                            extension=getOption("getSymbols.FI.extension", "rda"),
+                            split_method=getOption("getSymbols.FI.split_method",
+                                                   c("days", "common")),
+                            use_identifier=getOption("getSymbols.FI.use_identifier",
+                                                     NA),
+                            date_format=getOption("getSymbols.FI.date_format"),
+                            verbose=getOption("getSymbols.FI.verbose", TRUE),
+                            days_to_omit=getOption("getSymbols.FI.days_to_omit",
+                                                   c("Saturday", "Sunday")),
+                            indexTZ=getOption("getSymbols.FI.indexTZ", NA)
+                         )
 {
     if (is.null(date_format)) date_format <- "%Y.%m.%d"
     if (is.null(days_to_omit)) days_to_omit <- 'NULL'
@@ -399,8 +408,7 @@ getSymbols.FI <- function(Symbols,
         Symbol <- ifelse(is.na(instr_str), make.names(Symbols[[i]]), instr_str)
         ndc<-nchar(dir)
         if(substr(dir,ndc,ndc)=='/') dir <- substr(dir,1,ndc-1) #remove trailing forward slash
-        ssd <- strsplit(dir,"/")[[1]]
-        if (identical(character(0), ssd) || (!identical(character(0), ssd) && ssd[length(ssd)] != Symbol)) dir <- paste(dir,Symbol,sep="/")
+        dir <- paste(dir, Symbol, sep="/")
         
         if(!dir=="" && !file.exists(dir)) {
             if (verbose) cat("\ndirectory ",dir," does not exist, skipping\n")
@@ -448,7 +456,7 @@ getSymbols.FI <- function(Symbols,
                         }
                     } # end 'common'/default method (same as getSymbols.rda)    
                 ) # end split_method switch
-            fr <- quantmod:::convert.time.series(fr=fr,return.class=return.class)
+            fr <- convert.time.series(fr=fr,return.class=return.class)
             Symbols[[i]] <-make.names(Symbols[[i]]) 
             tmp <- list()
             tmp[[Symbols[[i]]]] <- fr
