@@ -8,7 +8,7 @@
 # This library is distributed under the terms of the GNU Public License (GPL)
 # for full details see the file COPYING
 #
-# $Id: ls_instruments.R 1209 2012-10-03 22:16:35Z gsee $
+# $Id: ls_instruments.R 1498 2013-08-25 00:26:39Z gsee $
 #
 ###############################################################################
 
@@ -118,12 +118,12 @@ ls_instruments <- function(pattern=NULL, match=TRUE, verbose=TRUE) {
         match <- TRUE    
     }    
     if (!is.null(pattern) && match) {   #there's a pattern and match is TRUE
-        symbols <- ls(FinancialInstrument:::.instrument, all.names=TRUE)
+        symbols <- ls(.instrument, all.names=TRUE)
         symbols <- symbols[match(pattern,symbols)]
     } else if (!match && length(pattern) == 1) { # pattern is length(1) and don't match
-        symbols <- ls(FinancialInstrument:::.instrument, all.names=TRUE, pattern=pattern)
+        symbols <- ls(.instrument, all.names=TRUE, pattern=pattern)
     } else if (is.null(pattern)) {  #no pattern
-        symbols <- ls(FinancialInstrument:::.instrument, all.names=TRUE)
+        symbols <- ls(.instrument, all.names=TRUE)
     } # else pattern length > 1 & don't match
     
     is.iname <- is.instrument.name(symbols)
@@ -137,7 +137,7 @@ ls_stocks <- function(pattern=NULL,match=TRUE) {
     symbols <- ls_instruments(pattern,match)
     tmp_symbols <- NULL            
     for (instr in symbols) {
-        tmp_instr <- try(get(instr, pos = FinancialInstrument:::.instrument),silent=TRUE)
+        tmp_instr <- try(get(instr, pos = .instrument),silent=TRUE)
         if (inherits(tmp_instr, 'stock') && inherits(tmp_instr, 'instrument')) {
             tmp_symbols <- c(tmp_symbols,instr)
         }    
@@ -151,7 +151,7 @@ ls_options <- function(pattern=NULL,match=TRUE, include.series=TRUE) {
     symbols <- ls_instruments(pattern,match)    
     tmp_symbols <- NULL            
     for (instr in symbols) {
-        tmp_instr <- try(get(instr, pos = FinancialInstrument:::.instrument),silent=TRUE)
+        tmp_instr <- try(get(instr, pos = .instrument),silent=TRUE)
         if (inherits(tmp_instr, 'option') && inherits(tmp_instr, 'instrument')) {
             if (!inherits(tmp_instr, 'option_series') || include.series)
                 tmp_symbols <- c(tmp_symbols,instr)
@@ -166,7 +166,7 @@ ls_option_series <- function(pattern=NULL,match=TRUE) {
     symbols <- ls_instruments(pattern,match)    
     tmp_symbols <- NULL            
     for (instr in symbols) {
-        tmp_instr <- try(get(instr, pos = FinancialInstrument:::.instrument),silent=TRUE)
+        tmp_instr <- try(get(instr, pos = .instrument),silent=TRUE)
         if (inherits(tmp_instr, 'option_series') && inherits(tmp_instr, 'instrument')) {
             tmp_symbols <- c(tmp_symbols,instr)
         }    
@@ -180,7 +180,7 @@ ls_futures <- function(pattern=NULL,match=TRUE, include.series=TRUE) {
     symbols <- ls_instruments(pattern,match)
     tmp_symbols <- NULL            
     for (instr in symbols) {
-        tmp_instr <- try(get(instr, pos = FinancialInstrument:::.instrument),silent=TRUE)
+        tmp_instr <- try(get(instr, pos = .instrument),silent=TRUE)
         if (inherits(tmp_instr, 'future') && inherits(tmp_instr, 'instrument')) {
             if (!inherits(tmp_instr, 'future_series') || include.series)
                 tmp_symbols <- c(tmp_symbols,instr)
@@ -195,7 +195,7 @@ ls_future_series <- function(pattern=NULL,match=TRUE) {
     symbols <- ls_instruments(pattern,match)
     tmp_symbols <- NULL            
     for (instr in symbols) {
-        tmp_instr <- try(get(instr, pos = FinancialInstrument:::.instrument),silent=TRUE)
+        tmp_instr <- try(get(instr, pos = .instrument),silent=TRUE)
         if (inherits(tmp_instr, 'future_series') && inherits(tmp_instr, 'instrument')) {
             tmp_symbols <- c(tmp_symbols,instr)
         }    
@@ -209,7 +209,7 @@ ls_currencies <- function(pattern=NULL, match=TRUE, includeFX=FALSE) {
     symbols <- ls_instruments(pattern=pattern, match=match)
     tmp_symbols <- NULL
     for (instr in symbols) {
-        tmp_instr <- try(get(instr, pos = FinancialInstrument:::.instrument),
+        tmp_instr <- try(get(instr, pos = .instrument),
                          silent=TRUE)
         if (inherits(tmp_instr, 'currency') 
             && inherits(tmp_instr, 'instrument')) {
@@ -227,7 +227,7 @@ ls_non_currencies <- function(pattern=NULL, match=TRUE, includeFX=TRUE) {
     symbols <- ls_instruments(pattern, match)
     tmp_symbols <- NULL            
     for (instr in symbols) {
-        tmp_instr <- try(get(instr, pos = FinancialInstrument:::.instrument),
+        tmp_instr <- try(get(instr, pos = .instrument),
                          silent=TRUE)
         if (!inherits(tmp_instr, 'currency') || 
                 (inherits(tmp_instr, 'exchange_rate') && includeFX) ) {
@@ -243,7 +243,7 @@ ls_exchange_rates <- function(pattern=NULL, match=TRUE) {
     symbols <- ls_currencies(pattern=pattern, match=match, includeFX=TRUE)
     tmp_symbols <- NULL            
     for (instr in symbols) {
-        tmp_instr <- try(get(instr, pos = FinancialInstrument:::.instrument),
+        tmp_instr <- try(get(instr, pos = .instrument),
                          silent=TRUE)
         if (inherits(tmp_instr, 'exchange_rate') 
             && inherits(tmp_instr, 'instrument')) {
@@ -263,7 +263,7 @@ ls_bonds <- function(pattern=NULL,match=TRUE) {
     symbols <- ls_instruments(pattern,match)
     tmp_symbols <- NULL            
     for (instr in symbols) {
-        tmp_instr <- try(get(instr, pos = FinancialInstrument:::.instrument),silent=TRUE)
+        tmp_instr <- try(get(instr, pos = .instrument),silent=TRUE)
         if (inherits(tmp_instr, 'bond') && inherits(tmp_instr, 'instrument')) {
             tmp_symbols <- c(tmp_symbols,instr)
         }    
@@ -277,7 +277,7 @@ ls_funds <- function(pattern=NULL,match=TRUE) {
     symbols <- ls_instruments(pattern,match)
     tmp_symbols <- NULL            
     for (instr in symbols) {
-        tmp_instr <- try(get(instr, pos = FinancialInstrument:::.instrument),silent=TRUE)
+        tmp_instr <- try(get(instr, pos = .instrument),silent=TRUE)
         if (inherits(tmp_instr, 'fund') && inherits(tmp_instr, 'instrument')) {
             tmp_symbols <- c(tmp_symbols,instr)
         }    
@@ -291,7 +291,7 @@ ls_spreads <- function(pattern=NULL,match=TRUE) {
     symbols <- ls_instruments(pattern,match)
     tmp_symbols <- NULL            
     for (instr in symbols) {
-        tmp_instr <- try(get(instr, pos = FinancialInstrument:::.instrument),silent=TRUE)
+        tmp_instr <- try(get(instr, pos = .instrument),silent=TRUE)
         if (inherits(tmp_instr, 'spread') && inherits(tmp_instr, 'instrument')) {
             tmp_symbols <- c(tmp_symbols,instr)
         }    
@@ -305,7 +305,7 @@ ls_guaranteed_spreads <- function(pattern=NULL,match=TRUE) {
     symbols <- ls_instruments(pattern,match)
     tmp_symbols <- NULL            
     for (instr in symbols) {
-        tmp_instr <- try(get(instr, pos = FinancialInstrument:::.instrument),silent=TRUE)
+        tmp_instr <- try(get(instr, pos = .instrument),silent=TRUE)
         if (inherits(tmp_instr, 'guaranteed_spread') && inherits(tmp_instr, 'instrument')) {
             tmp_symbols <- c(tmp_symbols,instr)
         }    
@@ -319,7 +319,7 @@ ls_synthetics <- function(pattern=NULL, match=TRUE) {
     symbols <- ls_instruments(pattern,match)    
     tmp_symbols <- NULL            
     for (instr in symbols) {
-        tmp_instr <- try(get(instr, pos = FinancialInstrument:::.instrument),silent=TRUE)
+        tmp_instr <- try(get(instr, pos = .instrument),silent=TRUE)
         if (inherits(tmp_instr, 'synthetic') && inherits(tmp_instr, 'instrument')) {
             tmp_symbols <- c(tmp_symbols,instr)
         }    
@@ -333,7 +333,7 @@ ls_ICS <- function(pattern=NULL, match=TRUE) {
     symbols <- ls_instruments(pattern,match)    
     tmp_symbols <- NULL            
     for (instr in symbols) {
-        tmp_instr <- try(get(instr, pos = FinancialInstrument:::.instrument),silent=TRUE)
+        tmp_instr <- try(get(instr, pos = .instrument),silent=TRUE)
         if (inherits(tmp_instr, 'ICS') && inherits(tmp_instr, 'instrument')) {
             tmp_symbols <- c(tmp_symbols,instr)
         }    
@@ -347,7 +347,7 @@ ls_ICS_roots <- function(pattern=NULL, match=TRUE) {
     symbols <- ls_instruments(pattern,match)
     tmp_symbols <- NULL            
     for (instr in symbols) {
-        tmp_instr <- try(get(instr, pos = FinancialInstrument:::.instrument),silent=TRUE)
+        tmp_instr <- try(get(instr, pos = .instrument),silent=TRUE)
         if (inherits(tmp_instr, 'ICS_root') && inherits(tmp_instr, 'instrument')) {
             tmp_symbols <- c(tmp_symbols,instr)
         }    
@@ -361,7 +361,7 @@ ls_ICS_roots <- function(pattern=NULL, match=TRUE) {
 #    symbols <- ls_instruments(pattern) #TODO: other functions should be updated to get symbols like this too   
 #    tmp_symbols <- NULL
 #    for (instr in symbols) {
-#        tmp_instr <- try(get(instr, pos = FinancialInstrument:::.instrument),silent=TRUE)
+#        tmp_instr <- try(get(instr, pos = .instrument),silent=TRUE)
 #        if ( is.instrument(tmp_instr) && !is.null(tmp_instr$defined.by) )  {        
 #            dby <- unlist(strsplit( tmp_instr$defined.by,";"))    
 #            if (any(dby == "yahoo" )) 
@@ -376,7 +376,7 @@ ls_ICS_roots <- function(pattern=NULL, match=TRUE) {
 #    symbols <- ls_instruments(pattern)   
 #    tmp_symbols <- NULL
 #    for (instr in symbols) {
-#        tmp_instr <- try(get(instr, pos = FinancialInstrument:::.instrument),silent=TRUE)
+#        tmp_instr <- try(get(instr, pos = .instrument),silent=TRUE)
 #        if ( is.instrument(tmp_instr) && !is.null(tmp_instr$defined.by) )  {        
 #            dby <- unlist(strsplit( tmp_instr$defined.by,";"))    
 #            if (any(dby == "IB" )) tmp_symbols <- c(tmp_symbols,instr)
@@ -390,7 +390,7 @@ ls_ICS_roots <- function(pattern=NULL, match=TRUE) {
 #	symbols <- ls_instruments(pattern)
 #	tmp_symbols <- NULL
 #	for (symbol in symbols) {
-#		tmp_instr <- try(get(symbol, pos=FinancialInstrument:::.instrument),silent=TRUE)
+#		tmp_instr <- try(get(symbol, pos=.instrument),silent=TRUE)
 #		if (is.instrument(tmp_instr) && !is.null(tmp_instr$defined.by) ) {
 #			dby <- unlist(strsplit( tmp_instr$defined.by,";"))
 #			if (any(dby == x)) tmp_symbols <- c(tmp_symbols,symbol)
@@ -407,7 +407,7 @@ ls_derivatives <- function(pattern=NULL,match=TRUE) {
     #but check for it in case someone made one    
     tmp_symbols <- NULL
     for (instr in symbols) {
-        tmp_instr <- try(get(instr, pos = FinancialInstrument:::.instrument),silent=TRUE)
+        tmp_instr <- try(get(instr, pos = .instrument),silent=TRUE)
          if (inherits(tmp_instr, 'derivative') || 
                  inherits(tmp_instr, 'option') ||
                  inherits(tmp_instr, 'future') ) {
@@ -425,7 +425,7 @@ ls_non_derivatives <- function(pattern=NULL,match=TRUE) {
     #but check for it in case someone made one    
     tmp_symbols <- NULL
     for (instr in symbols) {
-        tmp_instr <- try(get(instr, pos = FinancialInstrument:::.instrument),silent=TRUE)
+        tmp_instr <- try(get(instr, pos = .instrument),silent=TRUE)
          if (!inherits(tmp_instr, 'derivative') && 
                  !inherits(tmp_instr, 'option') &&
                  !inherits(tmp_instr, 'future') ) {
@@ -442,7 +442,7 @@ ls_calls <- function(pattern=NULL,match=TRUE) {
     symbols <- ls_options(pattern=pattern,match=match)
 	tmp_symbols <- NULL
     for (instr in symbols) {
-        tmp_instr <- try(get(instr, pos = FinancialInstrument:::.instrument),silent=TRUE)
+        tmp_instr <- try(get(instr, pos = .instrument),silent=TRUE)
 		if (is.instrument(tmp_instr) && inherits(tmp_instr, 'option')) {
 			if (!is.null(tmp_instr$callput)) {
 				right <- tmp_instr$callput
@@ -464,7 +464,7 @@ ls_puts <- function(pattern=NULL,match=TRUE) {
     symbols <- ls_options(pattern=pattern,match=match)
 	tmp_symbols <- NULL
     for (instr in symbols) {
-        tmp_instr <- try(get(instr, pos = FinancialInstrument:::.instrument),silent=TRUE)
+        tmp_instr <- try(get(instr, pos = .instrument),silent=TRUE)
 		if (is.instrument(tmp_instr) && inherits(tmp_instr, 'option')) {
 			if (!is.null(tmp_instr$callput)) {
 				right <- tmp_instr$callput
@@ -496,7 +496,7 @@ rm_instruments <- function(x, keep.currencies=TRUE) {
         } else stop('Use keep.currencies=FALSE to delete a currency')    
     }
 
-    rm(list=x,pos=FinancialInstrument:::.instrument)
+    rm(list=x,pos=.instrument)
 }
 
 #' @export
@@ -505,7 +505,7 @@ rm_stocks <- function(x) {
     if (missing(x)) {
         x <- ls_stocks()
     }
-    rm(list=x[x %in% ls_stocks()], pos=FinancialInstrument:::.instrument)
+    rm(list=x[x %in% ls_stocks()], pos=.instrument)
 }
 
 #' @export
@@ -514,7 +514,7 @@ rm_options <- function(x) {
     if (missing(x)) {
         x <- ls_options()
     }
-    rm(list=x[x %in% ls_options()], pos=FinancialInstrument:::.instrument)
+    rm(list=x[x %in% ls_options()], pos=.instrument)
 }
 
 #' @export
@@ -523,7 +523,7 @@ rm_option_series <- function(x) {
     if (missing(x)) {
         x <- ls_option_series()
     }
-    rm(list=x[x %in% ls_option_series()], pos=FinancialInstrument:::.instrument)
+    rm(list=x[x %in% ls_option_series()], pos=.instrument)
 }
 
 #' @export
@@ -532,7 +532,7 @@ rm_futures <- function(x) {
     if (missing(x)) {
         x <- ls_futures()
     }
-    rm(list=x[x %in% ls_futures()], pos=FinancialInstrument:::.instrument)
+    rm(list=x[x %in% ls_futures()], pos=.instrument)
 }
 
 #' @export
@@ -541,7 +541,7 @@ rm_future_series <- function(x) {
     if (missing(x)) {
         x <- ls_future_series()
     }
-    rm(list=x[x %in% ls_future_series()], pos=FinancialInstrument:::.instrument)
+    rm(list=x[x %in% ls_future_series()], pos=.instrument)
 }
 
 #' @export
@@ -550,7 +550,7 @@ rm_currencies <- function(x) {
     if (missing(x)) {
         x <- ls_currencies()
     }
-    rm(list=x[x %in% ls_currencies()], pos=FinancialInstrument:::.instrument)
+    rm(list=x[x %in% ls_currencies()], pos=.instrument)
 }   
 
 #' @export
@@ -559,7 +559,7 @@ rm_exchange_rates <- function(x) {
     if (missing(x)) {
         x <- ls_exchange_rates()
     }
-    rm(list=x[x %in% ls_exchange_rates()], pos=FinancialInstrument:::.instrument)
+    rm(list=x[x %in% ls_exchange_rates()], pos=.instrument)
 }
 
 #' @export
@@ -572,7 +572,7 @@ rm_bonds <- function(x) {
     if (missing(x)) {
         x <- ls_bonds()
     }
-    rm(list=x[x %in% ls_bonds()], pos=FinancialInstrument:::.instrument)
+    rm(list=x[x %in% ls_bonds()], pos=.instrument)
 }
 
 #' @export
@@ -581,7 +581,7 @@ rm_funds <- function(x) {
     if (missing(x)) {
         x <- ls_funds()
     }
-    rm(list=x[x %in% ls_funds()], pos=FinancialInstrument:::.instrument)
+    rm(list=x[x %in% ls_funds()], pos=.instrument)
 }
 
 #' @export
@@ -590,7 +590,7 @@ rm_spreads <- function(x) {
     if (missing(x)) {
         x <- ls_spreads()
     }
-    rm(list=x[x %in% ls_spreads()], pos=FinancialInstrument:::.instrument)
+    rm(list=x[x %in% ls_spreads()], pos=.instrument)
 }
 
 #' @export
@@ -599,7 +599,7 @@ rm_synthetics <- function(x) {
     if (missing(x)) {
         x <- ls_synthetics()
     }
-    rm(list=x[x %in% ls_synthetics()],pos=FinancialInstrument:::.instrument)
+    rm(list=x[x %in% ls_synthetics()],pos=.instrument)
 }
 
 
@@ -609,7 +609,7 @@ rm_derivatives <- function(x) {
     if (missing(x)) {
         x <- ls_derivatives()
     }
-    rm(list=x[x %in% ls_derivatives()],pos=FinancialInstrument:::.instrument)
+    rm(list=x[x %in% ls_derivatives()],pos=.instrument)
 }
 
 #' @export
@@ -624,7 +624,7 @@ rm_non_derivatives <- function(x, keep.currencies=TRUE) {
                 x <- x[-match(ls_currencies(),x)] #then take them out of to-be-removed
         } else stop('Use keep.currencies=FALSE to delete a currency')    
     }
-    rm(list=x[x %in% ls_non_derivatives()],pos=FinancialInstrument:::.instrument) 
+    rm(list=x[x %in% ls_non_derivatives()],pos=.instrument) 
 }
 
 
